import {attribute, DataStore, model, Model, Transport} from '@rx-suite/data';
import {t} from "@lingui/macro";
import * as yup from 'yup';
import {action, computed, observable} from "mobx";
import {DashboardComponentPlacementModel, DashboardComponentPlacementStore} from "./DashboardComponentPlacementStore";
import {Layout} from 'react-grid-layout'
import {DashboardComponentModel} from "./DashboardComponentStore";
import {computedFn} from "mobx-utils";

@model({
    transportLayer: new Transport("/api/dashboards", "dashboard"),
    permissionTarget: "DashboardComponent",
    routingInfo: {
        create: "/dashboards/create",
        update: "/dashboards/:id/edit",
        read: "/dashboards/:id/read"
    }
})
export class DashboardModel extends Model {
    @attribute({
        label: t`Name`,
        description: t`This is how you will identify the dashboard in lists (for example, you can search for this name).`,
        validation: yup.string().required(t`You must give your dashboard a name`)
    })
    name: string = '';

    @attribute({
        label: t`Description`,
        description: t`Add any notes here.  The information will not be included in the dashboard display - it is for reference purposes only.`,
        validation: yup.string()
    })
    desc: string = '';

    @observable
    tmpPlacements: DashboardComponentPlacementModel[] = [];

    @observable
    placementDeletionQueue: DashboardComponentPlacementModel[] = [];

    @computed
    get placements() {
        const typedID = parseInt(this.getId(), 10);
        return (DashboardComponentPlacementStore.models.filter(placement => {
            return (placement.viewModel.dashboard_id === typedID) && !(this.placementDeletionQueue.find(p => p.getId() === placement.getId()))
        }) as DashboardComponentPlacementModel[]).concat(this.tmpPlacements);
    }

    @computed
    get components() {
        return this.placements.map(p => p.viewModel.dashboard_component) as DashboardComponentModel[];
    }

    hasComponent = computedFn(function check(this: DashboardModel, component: DashboardComponentModel): any {
        return !!this.components.find(c => c && (c.getId() === component.getId()))
    });

    @action.bound
    handleLayoutChange(newLayouts: Layout[]) {
        newLayouts.forEach(layout => {
            const placement = this.placements.find(p => p.getId().toString() === layout.i);
            if (placement) {
                placement.handleLayoutChange(layout);
            }
        });
        window.dispatchEvent(new Event('resize')); // resize charts if user resized container
    }

    @action.bound
    addComponent(component: DashboardComponentModel) {
        const placement = DashboardComponentPlacementStore.createModel();
        placement.nestedAssignment('dashboard_id', parseInt(this.getId(), 10));
        placement.nestedAssignment('dashboard_component_id', parseInt(component.getId(), 10));
        placement.nestedAssignment('width', 12);
        placement.nestedAssignment('height', 12);
        this.tmpPlacements.push(placement);
        this.placementDeletionQueue = this.placementDeletionQueue.filter(p => p.getId() !== placement.getId());
    }

    @action.bound
    removeComponent(component: DashboardComponentModel) {
        this.placements.forEach(p => {
            if (p.viewModel.dashboard_component_id && (p.viewModel.dashboard_component_id.toString() === component.getId().toString())) {
                this.removePlacement(p);
            }
        });
    }

    @action.bound
    removePlacement(placement: DashboardComponentPlacementModel) {
        this.placements.forEach(p => {
            if (p.getId() === placement.getId()) {
                this.tmpPlacements = this.tmpPlacements.filter(pTmp => pTmp.getId() !== p.getId());
                if (!p.isNew) {
                    this.placementDeletionQueue.push(p);
                }
            }
        });
    }

    @action.bound
    async commit() {
        const resp = await super.commit();
        this.placements.forEach(p => {
            p.nestedAssignment('dashboard_id', resp.id);
            p.commit();
        });
        this.placementDeletionQueue.forEach(p => {
            p.delete();
        });
        this.tmpPlacements = [];
        return resp;
    }
}

export const DashboardStore = DashboardModel.store as DataStore<DashboardModel>;
